{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consectutively.
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses stypes;

const Maxspecies = 5;

PROCEDURE counts;
PROCEDURE processes(time:double;var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
PROCEDURE derivs(t:double;var statevalue:yValueArray;VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

implementation

uses frontend, sysutils;

{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end;

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,j,k,npar:integer;
 nm:string[1];
 nm2:string[2];
begin
ModelDef.modelname := 'MBL MEL';
ModelDef.versionnumber := '1.1.0';
ModelDef.timeunit := 'year';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate:=3+numspp*3;
{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is 20 characters. }
for i:= 1 to numspp do
   begin
      str(i:1,nm);
      with stat[3*(i-1)+1] do
       begin
         name := 'Sp. '+nm+' C';   units:='gC m-2ground';  symbol:='Bc' + nm;
       end;
      with stat[3*(i-1)+2] do
       begin
         name:='Sp. '+nm+' N';     units:='gN m-2ground';  symbol:='Bn' + nm;
       end;
      with stat[3*(i-1)+3] do
       begin
         name:='Sp. '+nm+' Vc';    units:='';              symbol:='Vc' + nm;
       end;
   end;

with stat[3*numspp+1] do
 begin
   name:='DC';             units:='gC m-2ground';          symbol:='DC';
 end;
with stat[3*numspp+2] do
 begin
   name:='DN';             units:='gN m-2ground';          symbol:='DN';
 end;
with stat[3*numspp+3] do
 begin
   name:='EN';             units:='gN m-2ground';          symbol:='EN';
 end;

{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
 {****************}  
NumSpProc:=16;
ModelDef.numprocess:=ModelDef.numstate + NumSpProc*numspp+34;

{ Set the names, units, and symbols of the processes and the parameters. The
  maximum length for the name, units and symbol is 20 characters.

  The first numstate processes are the derivatives of the state variables. Set
  the names and units accordingly.}
for i:= 1 to ModelDef.numstate do
  begin
    proc[i].name:='d' + stat[i].name + 'dt';
    proc[i].units := stat[i].units + 't-1';
    proc[i].symbol := 'd' + stat[i].symbol + 'dt';
    proc[i].parameters := 0;
    proc[i].Isdiscrete := false;
  end;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }

k:=ModelDef.numstate;
{***********************     BEGIN SP      ******************************}
for i:= 1 to numspp do
  begin
    str(i:1,nm);
    proc[NumSpProc*(i-1)+k+1].name := 'Act Tiss C Sp' + nm;
    proc[NumSpProc*(i-1)+k+1].units := 'gC m-2 grnd';
    proc[NumSpProc*(i-1)+k+1].symbol := 'BA' + nm;
    proc[NumSpProc*(i-1)+k+1].parameters := 6;
    proc[NumSpProc*(i-1)+k+1].Isdiscrete := false;
    npar:=ParCount(NumSpProc*(i-1)+k+1);
    with par[npar+1] do
     begin
       name:='Max Active Tiss Sp'+nm;    units := 'gC/m2 grnd';   symbol := 'Bamax' + nm;
     end;
    for j:= 2 to 6 do
      with par[npar+j] do
        begin
          name:='';    units := '';   symbol := '';
        end;
    for j:=1 to numspp do
      begin
        str(10*j+i,nm2);
        par[npar+j+1].name:='Allometry Eff ' + inttostr(j) + ' on ' + inttostr(i);
        par[npar+j+1].units := 'gC/gC';
        par[npar+j+1].symbol := 'C' + nm2;
      end;

    proc[NumSpProc*(i-1)+k+2].name := 'Opt. C:N Sp. ' + nm;
    proc[NumSpProc*(i-1)+k+2].units := 'gC/gN';
    proc[NumSpProc*(i-1)+k+2].symbol := 'q_opt' + nm;
    proc[NumSpProc*(i-1)+k+2].parameters := 2;
    proc[NumSpProc*(i-1)+k+2].Isdiscrete := false;
    npar:=ParCount(NumSpProc*(i-1)+k+2);
      par[npar+1].name:='Min C:N Sp'+nm;    par[npar+1].units := 'gC/gN';
      par[npar+1].symbol:='qmin'+nm;
      par[npar+2].name:='Max C:N Sp'+nm;    par[npar+2].units := 'gC/gN';
      par[npar+2].symbol:='qmax'+nm;

    proc[NumSpProc*(i-1)+k+3].name := 'C:N Sp. ' + nm;
    proc[NumSpProc*(i-1)+k+3].units := 'gC/gN';
    proc[NumSpProc*(i-1)+k+3].symbol := 'q' + nm;
    proc[NumSpProc*(i-1)+k+3].parameters := 0;
    proc[NumSpProc*(i-1)+k+3].Isdiscrete := false;

    proc[NumSpProc*(i-1)+k+4].name := 'LAI Sp. ' + nm;
    proc[NumSpProc*(i-1)+k+4].units := 'm2/m2';
    proc[NumSpProc*(i-1)+k+4].symbol := 'Sc' + nm;
    proc[NumSpProc*(i-1)+k+4].parameters := 1;
    proc[NumSpProc*(i-1)+k+4].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+4);
        par[npar+1].name:='LA /gC Sp'+nm;       par[npar+1].units := 'm2/gC';
        par[npar+1].symbol := 'bC' + nm;

    proc[NumSpProc*(i-1)+k+5].name := 'Root Length Sp. '+nm;
    proc[NumSpProc*(i-1)+k+5].units := 'm/m2';
    proc[NumSpProc*(i-1)+k+5].symbol := 'Sn' + nm;
    proc[NumSpProc*(i-1)+k+5].parameters := 1;
    proc[NumSpProc*(i-1)+k+5].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+5);
        par[npar+1].name:='Root Len/gC Sp' + nm;       par[npar+1].units := 'm/gC';
        par[npar+1].symbol := 'bN' + nm;

    proc[NumSpProc*(i-1)+k+6].name := 'Canopy Scaling Sp'+nm;
    proc[NumSpProc*(i-1)+k+6].units := '';
    proc[NumSpProc*(i-1)+k+6].symbol := 'wC' + nm;
    proc[NumSpProc*(i-1)+k+6].parameters := 1;
    proc[NumSpProc*(i-1)+k+6].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+6);
        par[npar+1].name:='Canopy Domin Sp'+nm;       par[npar+1].units := '';
        par[npar+1].symbol := 'fC' + nm;

    proc[NumSpProc*(i-1)+k+7].name := 'Root Scaling Sp. '+nm;
    proc[NumSpProc*(i-1)+k+7].units := '';
    proc[NumSpProc*(i-1)+k+7].symbol := 'wN';
    proc[NumSpProc*(i-1)+k+7].parameters := 1;
    proc[NumSpProc*(i-1)+k+7].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+7);
        par[npar+1].name:='Soil Domin Sp'+nm;       par[npar+1].units := '';
        par[npar+1].symbol := 'fN'+nm;

    proc[NumSpProc*(i-1)+k+8].name := 'GPP Sp ' + nm;
    proc[NumSpProc*(i-1)+k+8].units := 'gC/m2/yr';
    proc[NumSpProc*(i-1)+k+8].symbol := 'UCv' + nm;
    proc[NumSpProc*(i-1)+k+8].parameters := 2;
    proc[NumSpProc*(i-1)+k+8].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+8);
        par[npar+1].name:='Plant C Uptake Sp' + nm;  par[npar+1].units := 'gC/m2/yr';
        par[npar+1].symbol := 'gC' + nm;
        par[npar+2].name:='CO2 Half Sat Sp' + nm;    par[npar+2].units := 'ppmv';
        par[npar+2].symbol := 'kC' + nm;

    proc[NumSpProc*(i-1)+k+9].name := 'NPP Sp. ' + nm;
    proc[NumSpProc*(i-1)+k+9].units := 'gC/m2/yr';
    proc[NumSpProc*(i-1)+k+9].symbol := 'NPP Sp ' + nm;
    proc[NumSpProc*(i-1)+k+9].parameters := 0;
    proc[NumSpProc*(i-1)+k+9].Isdiscrete := false;

    proc[NumSpProc*(i-1)+k+10].name := 'N uptake Sp ' + nm;
    proc[NumSpProc*(i-1)+k+10].units := 'gN/m2/yr';
    proc[NumSpProc*(i-1)+k+10].symbol := 'UNv' + nm;
    proc[NumSpProc*(i-1)+k+10].parameters := 2;
    proc[NumSpProc*(i-1)+k+10].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+10);
        par[npar+1].name:='Plant N Uptake Sp'+nm;    par[npar+1].units := 'gN/m2/yr';
        par[npar+1].symbol:= 'gN'+nm;
        par[npar+2].name:='N Half Sat Sp'+nm;    par[npar+2].units := 'gN/m2';
        par[npar+2].symbol:= 'kN'+nm;

    proc[NumSpProc*(i-1)+k+11].name := 'C litter Sp '+nm;
    proc[NumSpProc*(i-1)+k+11].units := 'gC/m2/yr';
    proc[NumSpProc*(i-1)+k+11].symbol := 'LCv' + nm;
    proc[NumSpProc*(i-1)+k+11].parameters := 3;
    proc[NumSpProc*(i-1)+k+11].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+11);
        par[npar+1].name:='Act Tissue Lit Sp'+nm;    par[npar+1].units := '1/yr';
        par[npar+1].symbol := 'mA'+nm;
        par[npar+2].name:='Wood Lit Sp'+nm;    par[npar+2].units := '1/yr/g(sw-1)C';
        par[npar+2].symbol := 'mW'+nm;
        par[npar+3].name:='Wood exp Sp'+nm;    par[npar+3].units := '';
        par[npar+3].symbol := 'sW'+nm;

    proc[NumSpProc*(i-1)+k+12].name := 'N in litter Sp '+nm;
    proc[NumSpProc*(i-1)+k+12].units := 'gN/m2/yr';
    proc[NumSpProc*(i-1)+k+12].symbol := 'LNv'+nm;
    proc[NumSpProc*(i-1)+k+12].parameters := 2;
    proc[NumSpProc*(i-1)+k+12].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+12);
        par[npar+1].name:='C:N Act Tiss Lit Sp'+nm;    par[npar+1].units := 'gC/gN';
        par[npar+1].symbol:= 'qA'+nm;
        par[npar+2].name:='C:N Wood Lit Sp'+nm;    par[npar+2].units := 'gC/gN';
        par[npar+2].symbol:= 'qW'+nm;

    proc[NumSpProc*(i-1)+k+13].name := 'Respiration Sp'+nm;
    proc[NumSpProc*(i-1)+k+13].units := 'gC/m2/yr';
    proc[NumSpProc*(i-1)+k+13].symbol := 'RCv1';
    proc[NumSpProc*(i-1)+k+13].parameters := 2;
    proc[NumSpProc*(i-1)+k+13].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+13);
        par[npar+1].name:='Act Tiss Resp Sp'+nm;    par[npar+1].units := '1/yr';
        par[npar+1].symbol := 'rA'+nm;
        par[npar+2].name:='Wood Resp Sp'+nm;    par[npar+2].units := '1/yr';
        par[npar+2].symbol := 'rW'+nm;

    proc[NumSpProc*(i-1)+k+14].name := 'Acclim Pot Sp'+nm;
    proc[NumSpProc*(i-1)+k+14].units := '';
    proc[NumSpProc*(i-1)+k+14].symbol := 'A'+nm;
    proc[NumSpProc*(i-1)+k+14].parameters := 2;
    proc[NumSpProc*(i-1)+k+14].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+14);
        par[npar+1].name:='Acclim Rate Sp'+nm;    par[npar+1].units := '1/yr';
        par[npar+1].symbol := 'a'+nm;
        par[npar+2].name:='Acclim Damping Sp'+nm;    par[npar+2].units := 'yr';
        par[npar+2].symbol := 'h'+nm;

    proc[NumSpProc*(i-1)+k+15].name := 'Nfix'+nm;
    proc[NumSpProc*(i-1)+k+15].units := 'gN/m2/yr';
    proc[NumSpProc*(i-1)+k+15].symbol := 'Nfix'+nm;
    proc[NumSpProc*(i-1)+k+15].parameters := 3;
    proc[NumSpProc*(i-1)+k+15].Isdiscrete := false;
      npar:=ParCount(NumSpProc*(i-1)+k+15);
        par[npar+1].name:='Nfix Rate Sp'+nm;         par[npar+1].units := 'gN/m/yr';
        par[npar+1].symbol := 'gNfix'+nm;
        par[npar+2].name:='Nfix shutoff'+nm;         par[npar+2].units := '';
        par[npar+2].symbol := 'VNfix'+nm;
        par[npar+3].name:='Nfix C cost Sp'+nm;       par[npar+3].units := 'gC/gN';
        par[npar+3].symbol := 'rNfix'+nm;  

    proc[NumSpProc*(i-1)+k+16].name := 'costNup'+nm;
    proc[NumSpProc*(i-1)+k+16].units := '';
    proc[NumSpProc*(i-1)+k+16].symbol := 'costNup'+nm;
    proc[NumSpProc*(i-1)+k+16].parameters := 0;
    proc[NumSpProc*(i-1)+k+16].Isdiscrete := false;
   end;
{************************     END SP     **************************}
NumSpPar:=ParCount(k+NumSpProc+1);
proc[k+NumSpProc*numspp+1].name := 'Total veg C';
proc[k+NumSpProc*numspp+1].units := 'gC/m2';
proc[k+NumSpProc*numspp+1].symbol := 'BCT';
proc[k+NumSpProc*numspp+1].parameters := 0;
proc[k+NumSpProc*numspp+1].Isdiscrete := false;

proc[k+NumSpProc*numspp+2].name := 'Total veg N';
proc[k+NumSpProc*numspp+2].units := 'gN/m2';
proc[k+NumSpProc*numspp+2].symbol := 'BNT';
proc[k+NumSpProc*numspp+2].parameters := 0;
proc[k+NumSpProc*numspp+2].Isdiscrete := false;

proc[k+NumSpProc*numspp+3].name := 'Total GPP';
proc[k+NumSpProc*numspp+3].units := 'gC/m2/yr';
proc[k+NumSpProc*numspp+3].symbol := 'UCvT';
proc[k+NumSpProc*numspp+3].parameters := 0;
proc[k+NumSpProc*numspp+3].Isdiscrete := false;

proc[k+NumSpProc*numspp+4].name := 'Total veg resp';
proc[k+NumSpProc*numspp+4].units := 'gC/m2/yr';
proc[k+NumSpProc*numspp+4].symbol := 'RCvT';
proc[k+NumSpProc*numspp+4].parameters := 0;
proc[k+NumSpProc*numspp+4].Isdiscrete := false;

proc[k+NumSpProc*numspp+5].name := 'Total NPP';
proc[k+NumSpProc*numspp+5].units := 'gC/m2/yr';
proc[k+NumSpProc*numspp+5].symbol := 'NPP';
proc[k+NumSpProc*numspp+5].parameters := 0;
proc[k+NumSpProc*numspp+5].Isdiscrete := false;

proc[k+NumSpProc*numspp+6].name := 'Total Leaf Area';
proc[k+NumSpProc*numspp+6].units := 'm2/m2';
proc[k+NumSpProc*numspp+6].symbol := 'SCT';
proc[k+NumSpProc*numspp+6].parameters := 1;
proc[k+NumSpProc*numspp+6].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+6);
   par[npar+1].name:='Light Extinction';       par[npar+1].units := 'm2/m2';
   par[npar+1].symbol := 'Kl';

proc[k+NumSpProc*numspp+7].name := 'Total root Length';
proc[k+NumSpProc*numspp+7].units := 'm/m2';
proc[k+NumSpProc*numspp+7].symbol := 'SNT';
proc[k+NumSpProc*numspp+7].parameters := 3;
proc[k+NumSpProc*numspp+7].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+7);
   par[npar+1].name:='Soil Resource Ext';       par[npar+1].units := 'm2/m';
   par[npar+1].symbol := 'KS';        
   par[npar+2].name:='Soil Resource mod';       par[npar+2].units := 'm2/gN';
   par[npar+2].symbol := 'KSm';          
   par[npar+3].name:='Soil Resource Ext range'; par[npar+3].units := 'm2/m';
   par[npar+3].symbol := 'DKS';

proc[k+NumSpProc*numspp+8].name := 'veg Q10';
proc[k+NumSpProc*numspp+8].units := '';
proc[k+NumSpProc*numspp+8].symbol := 'Q10v';
proc[k+NumSpProc*numspp+8].parameters := 5;
proc[k+NumSpProc*numspp+8].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+8);
   par[npar+1].name:='Q10 on gC';   par[npar+1].units := '';
   par[npar+1].symbol := '';
   par[npar+2].name:='Q10 on gN';   par[npar+2].units := '';
   par[npar+2].symbol := '';
   par[npar+3].name:='Q10 on rA';   par[npar+3].units := '';
   par[npar+3].symbol := '';
   par[npar+4].name:='Q10 on rW';   par[npar+4].units := '';
   par[npar+4].symbol := '';    
   par[npar+5].name:='Q10 on gNfix';par[npar+5].units := '';
   par[npar+5].symbol := '';

proc[k+NumSpProc*numspp+9].name := 'N leaching';
proc[k+NumSpProc*numspp+9].units := 'gN/m2/yr';
proc[k+NumSpProc*numspp+9].symbol := 'LNE';
proc[k+NumSpProc*numspp+9].parameters := 2;
proc[k+NumSpProc*numspp+9].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+9);
   par[npar+1].name:='Inorg N Loss';       par[npar+1].units := 'gN^(1-gamma)/m^(2+2gamma)/yr';
   par[npar+1].symbol := 'betaNe';
   par[npar+2].name:='Inorg N Loss Exp';        par[npar+2].units := '';
   par[npar+2].symbol := 'gamma';

proc[k+NumSpProc*numspp+10].name := 'Soil C:N';
proc[k+NumSpProc*numspp+10].units := 'gC/gN';
proc[k+NumSpProc*numspp+10].symbol := 'qD';
proc[k+NumSpProc*numspp+10].parameters := 0;
proc[k+NumSpProc*numspp+10].Isdiscrete := false;

proc[k+NumSpProc*numspp+11].name := 'microb C:N consump';
proc[k+NumSpProc*numspp+11].units := 'gC/gN';
proc[k+NumSpProc*numspp+11].symbol := 'theta';
proc[k+NumSpProc*numspp+11].parameters := 2;
proc[k+NumSpProc*numspp+11].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+11);
   par[npar+1].name:='C:N microb byprod';       par[npar+1].units := 'gC/gN';
   par[npar+1].symbol := 'phi';
   par[npar+2].name:='Q10 on phi';       par[npar+2].units := '';
   par[npar+2].symbol := 'R10';

proc[k+NumSpProc*numspp+12].name := 'Microb. N uptake';
proc[k+NumSpProc*numspp+12].units := 'gN/m2/yr';
proc[k+NumSpProc*numspp+12].symbol := 'UNm';
proc[k+NumSpProc*numspp+12].parameters := 2;
proc[k+NumSpProc*numspp+12].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+12);
   par[npar+1].name:='Microb N Uptake/C';       par[npar+1].units := 'gN/gC';
   par[npar+1].symbol := 'alphaN';
   par[npar+2].name:='Microb N Half Sat';      par[npar+2].units := 'gN/m2';
   par[npar+2].symbol := 'kappaNm';

proc[k+NumSpProc*numspp+13].name := 'Tot mic N consump';
proc[k+NumSpProc*numspp+13].units := 'gN/m2/yr';
proc[k+NumSpProc*numspp+13].symbol := 'MUN';
proc[k+NumSpProc*numspp+13].parameters := 0;
proc[k+NumSpProc*numspp+13].Isdiscrete := false;

proc[k+NumSpProc*numspp+14].name := 'Micrb C efficiency';
proc[k+NumSpProc*numspp+14].units := '';
proc[k+NumSpProc*numspp+14].symbol := 'lambdaC';
proc[k+NumSpProc*numspp+14].parameters := 1;
proc[k+NumSpProc*numspp+14].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+14);
   par[npar+1].name:='Max Micrb C Effic.';       par[npar+1].units := '';
   par[npar+1].symbol := 'epsilonC';

proc[k+NumSpProc*numspp+15].name := 'Micrb N efficiency';
proc[k+NumSpProc*numspp+15].units := '';
proc[k+NumSpProc*numspp+15].symbol := 'lambdaN';
proc[k+NumSpProc*numspp+15].parameters := 1;
proc[k+NumSpProc*numspp+15].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+15);
   par[npar+1].name:='Max Micrb N Effic.';       par[npar+1].units := '';
   par[npar+1].symbol := 'epsilonN';

proc[k+NumSpProc*numspp+16].name := 'Microb. resp.';
proc[k+NumSpProc*numspp+16].units := 'gC/m2/yr';
proc[k+NumSpProc*numspp+16].symbol := 'RCm';
proc[k+NumSpProc*numspp+16].parameters := 2;
proc[k+NumSpProc*numspp+16].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+16);
   par[npar+1].name:='Decomp Rate';       par[npar+1].units := '1/yr';
   par[npar+1].symbol := 'psi';
   par[npar+2].name:='Q10 on psi';       par[npar+2].units := '';
   par[npar+2].symbol := 'Q10psi';

proc[k+NumSpProc*numspp+17].name := 'Gross N min.';
proc[k+NumSpProc*numspp+17].units := 'gN/m2/yr';
proc[k+NumSpProc*numspp+17].symbol := 'RNm';
proc[k+NumSpProc*numspp+17].parameters := 0;
proc[k+NumSpProc*numspp+17].Isdiscrete := false;

proc[k+NumSpProc*numspp+18].name := 'DOC loss';
proc[k+NumSpProc*numspp+18].units := 'gC/m2/yr';
proc[k+NumSpProc*numspp+18].symbol := 'LCD';
proc[k+NumSpProc*numspp+18].parameters := 1;
proc[k+NumSpProc*numspp+18].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+18);
   par[npar+1].name:='Organic C Loss';    par[npar+1].units := '1/yr';
   par[npar+1].symbol := 'betaCD';

proc[k+NumSpProc*numspp+19].name := 'DON loss';
proc[k+NumSpProc*numspp+19].units := 'gN/m2/yr';
proc[k+NumSpProc*numspp+19].symbol := 'LND';
proc[k+NumSpProc*numspp+19].parameters := 1;
proc[k+NumSpProc*numspp+19].Isdiscrete := false;
npar:=ParCount(k+NumSpProc*numspp+19);
   par[npar+1].name:='C:N leached OM';    par[npar+1].units := 'gC/gN';
   par[npar+1].symbol := 'eta';

proc[k+NumSpProc*numspp+20].name := 'Net N min.';
proc[k+NumSpProc*numspp+20].units := 'gN/m2/yr';
proc[k+NumSpProc*numspp+20].symbol := 'RNm-UNm';
proc[k+NumSpProc*numspp+20].parameters := 0;
proc[k+NumSpProc*numspp+20].Isdiscrete := false;

proc[k+NumSpProc*numspp+21].name := 'NEP';
proc[k+NumSpProc*numspp+21].units := 'gC/m2/yr';
proc[k+NumSpProc*numspp+21].symbol := 'UCvT-RCvT-RCm';
proc[k+NumSpProc*numspp+21].parameters := 0;
proc[k+NumSpProc*numspp+21].Isdiscrete := false;

with proc[k+NumSpProc*numspp+22] do
   begin
      name := 'free N fix';
      units := 'gN/m2/yr';
      symbol := 'Nfixm';
      parameters := 3;
      Isdiscrete := false;
   end;
npar:=ParCount(k+NumSpProc*numspp+22);
        par[npar+1].name:='free Nfix Rate';         par[npar+1].units := 'gN/gC/yr';
        par[npar+1].symbol := 'gNfixm';
        par[npar+2].name:='free Nfix shutoff';         par[npar+2].units := '';
        par[npar+2].symbol := 'qNfix';
        par[npar+3].name:='free Nfix C cost';       par[npar+3].units := 'gC/gN';
        par[npar+3].symbol := 'rNfixm';      

with proc[k+NumSpProc*numspp+23] do
   begin
      name := 'Total inorg. N';
      units := 'gN/m2';
      symbol := 'NT';
      parameters := 3;
      Isdiscrete := false;
   end;
npar:=ParCount(k+NumSpProc*numspp+23);
        par[npar+1].name:='Max sorbed N';         par[npar+1].units := 'gN/m2';
        par[npar+1].symbol := 'Nmax';
        par[npar+2].name:='slope';         par[npar+2].units := '';
        par[npar+2].symbol := 'beta0';
        par[npar+3].name:='fitting par';       par[npar+3].units := '';
        par[npar+3].symbol := 'gammaN';

{*******************}

{ And now do the discrete processes in the model. }

with proc[k+NumSpProc*numspp+24] do
   begin
      name := 'DBall';
      units := '';
      symbol := 'DBall';
      parameters := 3;
      Isdiscrete := true;
   end;
npar:=ParCount(k+NumSpProc*numspp+24);
  with par[npar+1] do begin
    name:='DB0all';     units := '';      symbol := 'DB0all';
   end;
  with par[npar+2] do begin
    name:='DBkall';     units := '';      symbol := 'DBkall';
   end;
  with par[npar+3] do begin
    name:='DBwt';       units := '';      symbol := 'DBwt';
   end;


for i:=1 to Maxspecies do
   begin
      str(i:1,nm);
      with proc[k+NumSpProc*numspp+24+i] do
         begin
            name := 'DB'+nm;
            units := '';
            symbol := 'DB'+nm;
            parameters := 3;
            Isdiscrete := true;
         end;
      npar:=ParCount(k+NumSpProc*numspp+24+i);
      with par[npar+1] do begin
         name:='DB0'+nm;    units := '';   symbol := 'DB0'+nm;
       end;
      with par[npar+2] do begin
         name:='DBk'+nm;    units := '';   symbol := 'DBk'+nm;
       end;  
      with par[npar+3] do begin
        name:='frac'+nm;    units := '';   symbol := 'frac'+nm;
       end;
   end;

for i:=1 to Maxspecies do
   begin
      str(i:1,nm);
      with proc[k+NumSpProc*numspp+29+i] do
         begin
            name := 'imm'+nm;
            units := '';
            symbol := 'IMM'+nm;
            parameters := 2;
            Isdiscrete := true;
         end;
      npar:=ParCount(k+NumSpProc*numspp+29+i);
      with par[npar+1] do begin
        name:='imm0'+nm;    units := '';   symbol := 'imm0'+nm;
       end;
      with par[npar+2] do begin
        name:='immk'+nm;    units := '';   symbol := 'immk'+nm;
       end;
   end;

{ Sum up the total number of parameters in the model and store it in the
  ModelDef structure.}
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

ModelDef.numdrive:=4;   {Maximum number of drivers is maxstate}
{Maximum length of driver name is 20 characters}
drive[1].name:='CO2';                 drive[1].units:='ppmv';
drive[1].symbol:='EC';
drive[2].name:='Air Temp';            drive[2].units:='deg C';
drive[2].symbol:='T';
drive[3].name:='Irradiance';          drive[3].units:='MJ m-2 day-1';
drive[3].symbol:='I';
drive[4].name:='N Deposition';        drive[4].units:='gN m-2 yr-1';
drive[4].symbol:='Rne';
end;

{ A function to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It accesses the global variables par,
  drive and ModelDef directly because those values are not modified.

  To simplify understanding the model equations all equations are written using
  variable names which correspond to the actual name instead of using the global
  arrays (i.e. SoilWater instead of stat[7].value). This makes it necessary to
  switch all values into local value, calculate everything and then put it all
  back into the global variables. Lengthy but worth it in terms of readability
  of the code.}
PROCEDURE processes(time:double;var tstat:statearray;var tproc:processarray;
                     CalculateDiscrete:Boolean);
var
// State variables
BC,BN,VC,dBC,dBN,dVC:array[1..Maxspecies] of double;
DC,DN,EN,dDC,dDN,dEN:double;

// Drivers
EC,RNe,Temp,Light:double;

// Species parameters
c:array[1..Maxspecies,1..Maxspecies] of double;
BAmax,qmin,qmax,bsC,bsN,fC,fN,gC,gN,kC,kN,mA,mW,sW,qA,qW,rA,rW,a,h,gNfix,
      VNfix,rNfix:array[1..Maxspecies] of double;
KI,KS,KSm,DKS,Q10gC,Q10gN,Q10rA,Q10rW,Q10gNfix:double;

// Soil parameters
betaNe,gamma,phi,R10phi,alphaN,kappaNm,epsilonC,epsilonN,psi,Q10psi,psiT,
    betaCD,eta,gNfixm,qNfix,rNfixm:double;

// extinction parameters
DB0,DBk,frac:array[1..Maxspecies] of double;
DB0all,DBkall,DBwt:double;

// immigration parameters
imm0,immk:array[1..Maxspecies] of double;

//soil exchange parameters
Nmax,beta0,gammaN:double;

//processes
BA,q,SC,SN,wC,wN,UCv,UNv,LCv,LNv,RCv,Aclm,Nfix,costNup:array[1..Maxspecies] of double;
SCT,SNT,LNe,THETA,UNm,MUN,LAMBDAC,LAMBDAN,RCm,RNm,SGPP,SNPP,SBC,SBN,
SRv,LND,LCD,dumC,dumN,Nfixm,NT:double;

//other
dum,kill,rndnm,rndnm2,SwC,SwN,dNTdN:double;

i,j,k,npar:integer;

function Q10calc(Q10:double):double;
begin
   if Q10>0 then Q10calc:=exp(ln(Q10)*temp/10)
   else Q10calc:=1;
end;


function rnd(a,k:double):double;
var dum:double;
begin
   
   if a>=1 then dum:=0
   else
      begin
         if a>=0 then
            begin
               if k<0 then
                  begin
                     dum:=1-(1-exp(k*(1-random)))/(1-exp(k*(1-a)));
                  end
               else
                  begin
                     dum:=k*(random-a)/(1-a);
                  end;
            end
         else
            begin
               if k<0 then
                  begin
                     dum:=1-(1-exp(k*(1-rndnm2)))/(1-exp(k*(1+a)));
                  end
               else
                  begin
                     dum:=k*(rndnm2+a)/(1+a);
                  end;
            end
      end;
   if dum<0 then rnd:=0
   else rnd:=dum;
end;

function UP(g,Q10,E,w,L,ki,ST,kC,SW:double):double;
begin
   if sw>0 then UP:=g*Q10calc(Q10)*E*w*L*(1-exp(-KI*ST))/((kC+E)*Sw) else UP:=0;
end;

Function Nupcost(i:integer):double;
var
   dV,dUC,dUN,cost,
   SC0,SN0,VC0,VN0,wc0,wn0,swC0,SwN0,SCT0,SNT0,UC0,UN0,
   SC1,SN1,VC1,VN1,wc1,wn1,swC1,SwN1,SCT1,SNT1,UC1,UN1:double;

   ii:integer;

begin
   VC0:=VC[i];
   VN0:=1-VC0;
   SC0:=SC[i];
   SN0:=SN[i];
   wc0:=wc[i];
   wN0:=wN[i];
   SWC0:=SWC;
   SWN0:=SWN;
   SCT0:=SCT;
   SNT0:=SNT;
   dV:=0.001;
   cost:=0;
   UC0:=UP(gC[i],Q10gc,EC,wC0,Light,ki,SCT0,kC[i],SWC0);
   UN0:=UP(gN[i],Q10gN,EN,wN0    ,1,kS,SNT0,kN[i],SWN0);
   for ii:=1 to 2 do
      begin
         VC1:=VC0+dV; if VC1>1 then VC1:=1; if VC1<0 then VC1:=0;
         VN1:=1-VC1;
         if VC0 > 0 then SC1:=SC0*VC1/VC0 else SC1:=0;
         if VN0 > 0 then SN1:=SN0*VN1/VN0 else SN1:=0;
         if exp(-kI*SC0)<1 then wC1:=wC0*(1-exp(-kI*SC1))/(1-exp(-kI*SC0)) else wC1:=wc0;
         if exp(-kS*SN0)<1 then wN1:=wN0*(1-exp(-kS*SN1))/(1-exp(-kS*SN0)) else wN1:=wN0;
         SWC1:=SWC0-wC0+wC1;
         SWN1:=SWN0-wN0+wN1;
         SCT1:=SCT0-sC0+sC1;
         SNT1:=SNT0-sN0+sN1;
         UC1:=UP(gC[i],Q10gc,EC,wC1,Light,ki,SCT1,kC[i],SWC1);
         UN1:=UP(gN[i],Q10gN,EN,wN1    ,1,kS,SNT1,kN[i],SWN1);
         dUC:=(UC1-UC0)/dV;
         dUN:=(UN1-UN0)/dV;
         if abs(dUN)>0 then cost:=cost-dUC/dUN;
         dV:=-dV;
      end;
    Nupcost:=0.5*cost;
end;

begin
{ Putting drivers into local variables }
EC    := drive[1].value;
Temp  := drive[2].value;
Light := drive[3].value;
RNe   := drive[4].value;


{ Putting state variables into local variables }
for i:= 1 to ModelDef.numstate do
  if tstat[i].value<=0 then tstat[i].value:=0;
for i:=1 to numspp do
   begin
      BC[i]:= tstat[3*(i-1)+1].value;
      BN[i]:= tstat[3*(i-1)+2].value;
      if (BC[i]<0.001) or (BN[i]<0.00005) then begin BC[i]:=0; BN[i]:=0; end;
      VC[i]:= tstat[3*(i-1)+3].value;
      if VC[i]>1 then VC[i]:=1;
      if VC[i]<0 then VC[i]:=0;
   end;
DC:= tstat[3*numspp+1].value;
DN:= tstat[3*numspp+2].value;
EN:= tstat[3*numspp+3].value;


{ And now parameters into local variables }
for i:= 1 to numspp do
   begin
      BAmax[i]:= par[NumSpPar*(i-1)+1].value;
      for j:= 1 to numspp do c[j,i] := par[NumSpPar*(i-1)+1+j].value;
      qmin[i] := par[NumSpPar*(i-1)+7].value;
      qmax[i] := par[NumSpPar*(i-1)+8].value;
      bsC[i]  := par[NumSpPar*(i-1)+9].value;
      bsN[i]  := par[NumSpPar*(i-1)+10].value;
      fC[i]   := par[NumSpPar*(i-1)+11].value;
      fN[i]   := par[NumSpPar*(i-1)+12].value;
      gC[i]   := par[NumSpPar*(i-1)+13].value;
      kC[i]   := par[NumSpPar*(i-1)+14].value;
      gN[i]   := par[NumSpPar*(i-1)+15].value;
      kN[i]   := par[NumSpPar*(i-1)+16].value;
      mA[i]   := par[NumSpPar*(i-1)+17].value;
      mW[i]   := par[NumSpPar*(i-1)+18].value;
      sW[i]   := par[NumSpPar*(i-1)+19].value;
      qA[i]   := par[NumSpPar*(i-1)+20].value;
      qW[i]   := par[NumSpPar*(i-1)+21].value;
      rA[i]   := par[NumSpPar*(i-1)+22].value;
      rW[i]   := par[NumSpPar*(i-1)+23].value;
      a[i]    := par[NumSpPar*(i-1)+24].value;
      h[i]    := par[NumSpPar*(i-1)+25].value;
      gNfix[i]:= par[NumSpPar*(i-1)+26].value;
      VNfix[i]:= par[NumSpPar*(i-1)+27].value;
      rNfix[i]:= par[NumSpPar*(i-1)+28].value;
   end;

npar    :=ParCount(modeldef.numstate+numspp*NumSpProc+1);
KI      :=par[npar+1].value;
KS      :=par[npar+2].value;
KSm     :=par[npar+3].value; 
DKS     :=par[npar+4].value;
Q10gC   :=par[npar+5].value;
Q10gN   :=par[npar+6].value;
Q10rA   :=par[npar+7].value;
Q10rW   :=par[npar+8].value;
Q10gNfix:=par[npar+9].value;
betaNe  :=par[npar+10].value;
gamma   :=par[npar+11].value;
phi     :=par[npar+12].value;
R10phi  :=par[npar+13].value;
alphaN  :=par[npar+14].value;
kappaNm :=par[npar+15].value;
epsilonC:=par[npar+16].value;
epsilonN:=par[npar+17].value;
psi     :=par[npar+18].value;
Q10psi  :=par[npar+19].value;
betaCD  :=par[npar+20].value;
eta     :=par[npar+21].value;
gNfixm  :=par[npar+22].value;
qNfix   :=par[npar+23].value;
rNfixm  :=par[npar+24].value;
Nmax    :=par[npar+25].value;
beta0   :=par[npar+26].value;
gammaN  :=par[npar+27].value;
DB0all  :=par[npar+28].value;
DBkall  :=par[npar+29].value;
DBwt    :=par[npar+30].value;

for i:= 1 to Maxspecies do
   begin
      DB0[i]:=par[npar+30+(i-1)*3+1].value;
      DBk[i]:=par[npar+30+(i-1)*3+2].value;
      frac[i]:=par[npar+30+(i-1)*3+3].value;
   end;

for i:= 1 to Maxspecies do
   begin
      imm0[i]:=par[npar+45+(i-1)*2+1].value;
      immk[i]:=par[npar+45+(i-1)*2+2].value;
   end;

//plant processes
SCT:=0;
SNT:=0;
SBC:=0;

// adjust KS to account for volume of fertile sites in soil
KS      :=KS+DKS*exp(-KSm*DN);

for i:= 1 to numspp do
   begin
      if (BC[i]>0) and (BN[i]>0)  then
         begin
            dum:=0;
            for j:= 1 to numspp do dum:=dum+c[j,i]*BC[j];
            BA[i]:=BAmax[i]*c[i,i]*BC[i]/(BAmax[i]+ dum);
            q[i]:=BC[i]*qmin[i]*qmax[i]/(BA[i]*qmax[i]+(BC[i]-BA[i])*qmin[i]);
            SC[i]:=bsC[i]*VC[i]*BA[i];
            SCT:=SCT+SC[i];
            SN[i]:=bsN[i]*(1-VC[i])*BA[i];
            SNT:=SNT+SN[i];
            SBC:=SBC+BC[i];
         end
      else
         begin

            BA[i]:=0;
            q[i]:=1;
            SC[i]:=0;
            SN[i]:=0;
         end;
   end;
SwC:=0;
SwN:=0;
for i:= 1 to numspp do
   begin
      if (BC[i]>0) and (BN[i]>0)  then
         begin
            dum:=1;
            for j:= 1 to numspp do
               if BC[j]>0 then dum:=dum*(fC[i]+fC[j]*exp(-KI*SC[j]))/(fC[i]+fC[j]);
            wC[i]:= exp(-KI*SC[i]);
            wC[i]:= 2.0*(1-wC[i])*dum/(1+wC[i]);
            SwC:=SwC+wC[i];
            dum:=1;
            for j:= 1 to numspp do
               if BC[j]>0 then dum:=dum*(fN[i]+fN[j]*exp(-KS*SN[j]))/(fN[i]+fN[j]);
            wN[i]:= exp(-KS*SN[i]);
            wN[i]:= 2.0*(1-wN[i])*dum/(1+wN[i]);
            SwN:=SwN+wN[i];
         end
      else
         begin
            wC[i]:=0;
            wN[i]:=0;
         end;
   end;
for i:= 1 to numspp do
   begin
      if (BC[i]>0) and (BN[i]>0) then
         begin
            if SwC>0 then UCv[i]:=UP(gC[i],Q10gc,EC,wC[i],Light,ki,SCT,kC[i],SWC)
            else UCv[i]:=0;
            if SwN>0 then UNv[i]:=UP(gN[i],Q10gN,EN,wN[i],1,kS,SNT,kN[i],SWN)
            else UNv[i]:=0;
{            if (gNfix[i]>0) and (VNfix[i]<0) then findVNfix(i,UCv[i],UNv[i],VNfix[i]);
            if VNfix[i]<0 then VNfix[i]:=0;
            Nfix[i]:=gNfix[i]*bsN[i]*BA[i]*(1-VC[i]-VNfix[i])*Q10calc(Q10gNfix);
            if Nfix[i]<0 then Nfix[i]:=0; }
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$
            if gNfix[i] = 0 then
               begin
                 Nfix[i]:=0;
                 costNup[i]:=0;
               end
            else
               begin
                  if VNfix[i]>=0 then
                     begin
                        costNup[i]:=0;
                        Nfix[i]:=gNfix[i]*bsN[i]*BA[i]*(1-VC[i]-VNfix[i])*Q10calc(Q10gNfix);
                     end
                  else
                     begin
                        costNup[i]:=Nupcost(i);
                        Nfix[i]:=(costNup[i]-rNfix[i]);
                        Nfix[i]:=Nfix[i]/(abs(VNfix[i])+Nfix[i]);
                        Nfix[i]:=gNfix[i]*bsN[i]*BA[i]*(1-VC[i])*Nfix[i]*Q10calc(Q10gNfix);
                     end;
                  if Nfix[i]<0 then Nfix[i]:=0;
               end;
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
            LCv[i]:=mA[i]*BA[i]+mW[i]*(BC[i]-Ba[i])*exp(ln(SBC)*sW[i]);
            RCv[i]:=rA[i]*BA[i]*Q10calc(Q10rA)+rW[i]*(BC[i]-Ba[i])*Q10calc(Q10rW);
            RCv[i]:=RCv[i]+Nfix[i]*rNfix[i];
            LNv[i]:=(mA[i]*BA[i]/qA[i]+mW[i]*(BC[i]-Ba[i])*exp(ln(SBC)*sW[i])/qW[i]);
            LNv[i]:=LNv[i]*q[i]*BN[i]/BC[i];
            dBC[i]:=UCv[i]-RCv[i]-LCv[i];
            dBN[i]:=UNv[i]+Nfix[i]-LNv[i];
            Aclm[i]:=ln(BC[i]/(q[i]*BN[i]));
            Aclm[i]:=Aclm[i]+ h[i]*(dBC[i]/BC[i]-dBN[i]/BN[i]);
            if Aclm[i]>0 then dVC[i]:=-a[i]*Aclm[i]*VC[i]
            else dVC[i]:=-a[i]*Aclm[i]*(1-VC[i]);
         end
      else
         begin
            UCv[i]:=0;
            UNv[i]:=0;
            LCv[i]:=0;
            LNv[i]:=0;
            RCv[i]:=0;
            dBC[i]:=0;
            dBN[i]:=0;
            Nfix[i]:=0;
            Aclm[i]:=0;
            dVC[i]:=0;
         end;
   end;

{soil processes}
psiT:=psi*Q10calc(Q10psi);
if EN>0 then LNe:=betaNe*exp(ln(EN)*gamma) else LNe:=0;
if epsilonC>0 then THETA:=epsilonN*phi*Q10calc(R10phi)/epsilonC else THETA:=8;
if DN>0 then
   begin
      UNm:=psiT*alphaN*sqr(DC)*EN/(THETA*DN*(kappaNm+EN));
      if EN>0 then Nfixm:=psiT*gNfixm*sqr(DC)*ln(qNfix/EN)/(theta*DN) else Nfixm:=100;
      if Nfixm<0 then Nfixm:=0;
   end
else
   begin
      UNm:=100;
      Nfixm:=10;
   end;


MUN:=psiT*DN+UNm+Nfixm;
if MUN>0 then
   begin
      LAMBDAC:=epsilonC*THETA*MUN/(psiT*DC+THETA*MUN);
      LAMBDAN:=epsilonN*psiT*DC/(psiT*DC+THETA*MUN);
   end;
RCm:=psiT*DC*(1-LAMBDAC) + Nfixm*rNfixm;
RNm:=MUN*(1-LAMBDAN);
LCD:=betaCD*DC;
If eta>0 then LND:=LCD/eta else LND:=0;

if EN = 0 then
   begin
      NT:=0;
      dNTdN:=1;
   end
else
   begin
      if Nmax>0 then
         begin
            dum := exp(ln(beta0*EN)*gammaN)+ exp(ln(Nmax)*gammaN);
            dum := exp(ln(dum)/gammaN); 
            NT := EN + beta0*Nmax*EN/dum; 
            dNTdN:=1 + (beta0*Nmax/dum) -  (sqr(beta0)*gammaN*Nmax*exp(ln(EN)*gammaN)/sqr(dum));
         end
      else
         begin
             NT := EN;
             dNTdN:=1;
         end;
   end;

dDC:=-RCm-LCD;
dDN:=UNm+Nfixm-RNm-LND;
dEN:=RNe+RNm-LNe-UNm;


for i := 1 to numspp do
   begin
      dDC:=dDC+LCv[i];
      dDN:=dDN+LNv[i];
      dEN:=dEN-UNv[i];
   end;
 
dEN:=dEN/dNTdN;
// Assign everything back into process variable array
// Derivatives
for i:= 1 to numspp do
   begin
      k:=3*(i-1);
      if tstat[k+1].HoldConstant = False then tproc[k+1].value:=dBC[i]
      else tproc[k+1].value:=0;
      if tstat[k+2].HoldConstant = False then tproc[k+2].value:=dBN[i]
      else tproc[k+2].value:=0;
      if tstat[k+3].HoldConstant = False then tproc[k+3].value:=dVC[i]
      else tproc[k+3].value:=0;
   end;

if tstat[3*numspp+1].HoldConstant = False then tproc[3*numspp+1].value:=dDC
else tproc[3*numspp+1].value:=0;
if tstat[3*numspp+2].HoldConstant = False then tproc[3*numspp+2].value:=dDN
else tproc[3*numspp+2].value:=0;
if tstat[3*numspp+3].HoldConstant = False then tproc[3*numspp+3].value:=dEN
else tproc[3*numspp+3].value:=0;

//other procs
k:=ModelDef.numstate;
SGPP:=0;SNPP:=0;SBC:=0;SBN:=0;SRv:=0;
for i:= 1 to numspp do
   begin
      tproc[NumSpProc*(i-1)+k+1 ].value:=BA[i];
      tproc[NumSpProc*(i-1)+k+2 ].value:=q[i];
      if BN[i]>0 then tproc[NumSpProc*(i-1)+k+3 ].value:=BC[i]/BN[i]
      else tproc[NumSpProc*(i-1)+k+3 ].value:=1;
      tproc[NumSpProc*(i-1)+k+4 ].value:=SC[i];
      tproc[NumSpProc*(i-1)+k+5 ].value:=SN[i];
      tproc[NumSpProc*(i-1)+k+6 ].value:=wC[i];
      tproc[NumSpProc*(i-1)+k+7 ].value:=wN[i];
      tproc[NumSpProc*(i-1)+k+8 ].value:=UCv[i];
      tproc[NumSpProc*(i-1)+k+9 ].value:=UCv[i]-RCv[i];
      tproc[NumSpProc*(i-1)+k+10].value:=UNv[i];
      tproc[NumSpProc*(i-1)+k+11].value:=LCv[i];
      tproc[NumSpProc*(i-1)+k+12].value:=LNv[i];
      tproc[NumSpProc*(i-1)+k+13].value:=RCv[i];
      tproc[NumSpProc*(i-1)+k+14].value:=Aclm[i];
      tproc[NumSpProc*(i-1)+k+15].value:=Nfix[i];  
      tproc[NumSpProc*(i-1)+k+16].value:=costNup[i];
      SGPP:=SGPP+UCv[i];
      SNPP:=SNPP+UCv[i]-RCv[i];
      SBC:=SBC+BC[i];
      SBN:=SBN+BN[i];
      SRv:=SRv+RCv[i];
   end;
tproc[k+NumSpProc*numspp+1].value:=SBC;
tproc[k+NumSpProc*numspp+2].value:=SBN;
tproc[k+NumSpProc*numspp+3].value:=SGPP;
tproc[k+NumSpProc*numspp+4].value:=SRv;
tproc[k+NumSpProc*numspp+5].value:=SNPP;
tproc[k+NumSpProc*numspp+6].value:=SCT;
tproc[k+NumSpProc*numspp+7].value:=SNT;
tproc[k+NumSpProc*numspp+8].value:=0;
tproc[k+NumSpProc*numspp+9].value:=LNe;
if DN>0 then tproc[k+NumSpProc*numspp+10].value:=DC/DN
else tproc[k+NumSpProc*numspp+10].value:=1;
tproc[k+NumSpProc*numspp+11].value:=THETA;
tproc[k+NumSpProc*numspp+12].value:=UNm;
tproc[k+NumSpProc*numspp+13].value:=MUN;
tproc[k+NumSpProc*numspp+14].value:=LAMBDAC;
tproc[k+NumSpProc*numspp+15].value:=LAMBDAN;
tproc[k+NumSpProc*numspp+16].value:=RCm;
tproc[k+NumSpProc*numspp+17].value:=RNm;
tproc[k+NumSpProc*numspp+18].value:=LCD;
tproc[k+NumSpProc*numspp+19].value:=LND;
tproc[k+NumSpProc*numspp+20].value:=RNm-UNm;
tproc[k+NumSpProc*numspp+21].value:=SNPP-RCm;  
tproc[k+NumSpProc*numspp+22].value:=Nfixm;  
tproc[k+NumSpProc*numspp+23].value:=NT;
{********************}

if CalculateDiscrete then
begin
 rndnm:=rnd(DB0all,DBkall);
 tproc[k+NumSpProc*numspp+24].value:=rndnm;

 rndnm2:=random;

 for i:= 1 to numspp do
   begin
      if DBkall <0 then kill:=rndnm*BC[i]
      else              kill:=rndnm;

     dum:=rnd(DB0[i],DBk[i]);

      if DBk[i]<0 then tproc[k+NumSpProc*numspp+24+i].value:=
                                         DBwt*kill+(1-DBwt)*dum*BC[i]
      else  tproc[k+NumSpProc*numspp+24+i].value:=DBwt*kill+(1-DBwt)*dum;
      if tproc[k+NumSpProc*numspp+24+i].value>BC[i] then
             tproc[k+NumSpProc*numspp+24+i].value:=BC[i];
   end;

 for i:= 1 to numspp do
   begin
      if immk[i]<0 then immk[i]:=-immk[i];
      if stat[3*(i-1)+1].value < 1000 then tproc[k+NumSpProc*numspp+29+i].value:=rnd(imm0[i],immk[i])
      else tproc[k+NumSpProc*numspp+29+i].value:=rnd(imm0[i],immk[i])*exp(-0.005*(stat[3*(i-1)+1].value-1000));
   end;
  for j:= 1 to numspp do
    begin
      dumC:= proc[modeldef.numstate+numspproc*numspp+24+j].value;
      if stat[3*(j-1)+1].value>0 then
         dumN:= dumC* stat[3*(j-1)+2].value/stat[3*(j-1)+1].value
      else begin dumC:=0; dumN:=0; end;
      stat[3*(j-1)+1].value:=stat[3*(j-1)+1].value-dumC;
      stat[3*(j-1)+2].value:=stat[3*(j-1)+2].value-dumN;
      stat[3*numspp+1].value:=stat[3*numspp+1].value+dumC*frac[j];
      stat[3*numspp+2].value:=stat[3*numspp+2].value+dumN*frac[j];

      dumC:=proc[modeldef.numstate+NumSpProc*numspp+29+j].value;
      dumN:=proc[modeldef.numstate+NumSpProc*numspp+29+j].value
                    /par[NumSpPar*(j-1)+7].value;
      stat[3*(j-1)+1].value:=stat[3*(j-1)+1].value+dumC;
      stat[3*(j-1)+2].value:=stat[3*(j-1)+2].value+dumN;
    {  if (stat[3*(j-1)+3].value<0) or
           (stat[3*(j-1)+3].value>1) then stat[3*(j-1)+3].value:=0.5; }
      stat[3*(j-1)+3].value:=0.5*dumC + (stat[3*(j-1)+1].value-dumC)*stat[3*(j-1)+3].value;
      stat[3*(j-1)+3].value:=stat[3*(j-1)+3].value/stat[3*(j-1)+1].value;
    end;
end;

end;

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not their names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t:double;var statevalue:yValueArray;
                                VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t,tempstate,tempproc,false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;

end.
 