{ This unit contains the type declaration for two new objects, TMyTabSheet and
TMyPageControl. The TMyPageControl is identical to the Delphi object except it
contains TMyTabSheets instead of TTabSheets. TMyTabSheet differs from TTabSheet
in that it contains these objects; a panel, three labels and a stringgrid.
 This file was copied out of the Delphi component library so it should not be
necessary to edit the code. The sections which differ from the Delphi
component library are marked by comments, search for the word 'added' to easily
find those sections.
 If you are encountering errors which appear to point to code in this unit the
most probable cause is that an invalid parameter is being passed to the
PageControl or the TabSheet.
}
unit newcontrols;

interface

uses
  Windows, Messages, SysUtils, CommCtrl, Classes, Graphics, Controls, Forms,
  StdCtrls, Grids, Comctrls, {Next units added by BK} extctrls, AdvGrid;

type
  TControlForm = class(TForm)
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TMyPageControl = class;    { Forward Declaration }

  TMyTabSheet = class(TWinControl)
  private
    FPageControl: TMyPageControl;
    FTabVisible: Boolean;
    FTabShowing: Boolean;
    function GetPageIndex: Integer;
    function GetTabIndex: Integer;
    procedure SetPageControl(APageControl: TMyPageControl);
    procedure SetPageIndex(Value: Integer);
    procedure SetTabShowing(Value: Boolean);
    procedure SetTabVisible(Value: Boolean);
    procedure UpdateTabShowing;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure ReadState(Reader: TReader); override;
  public
// Code added by BK to add labels and a stringgrid to the TabSheet.  15Dec97
    PnTop: TPanel;
    LblProcessValue: TLabel;
    LblProcessName: TLabel;
    LblProcessUnits: TLabel;
    SgParams: TStringGrid;
// End of added code
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property PageControl: TMyPageControl read FPageControl write SetPageControl;
    property TabIndex: Integer read GetTabIndex;
  published
    property Caption;
    property Enabled;
    property Font;
    property Height stored False;
    property Left stored False;
    property PageIndex: Integer read GetPageIndex write SetPageIndex stored False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabVisible: Boolean read FTabVisible write SetTabVisible default True;
    property Top stored False;
    property Visible stored False;
    property Width stored False;
    property OnDragDrop;
    property OnDragOver;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TMyPageControl = class(TCustomTabControl)
  private
    FPages: TList;
    FActivePage: TMyTabSheet;
    procedure ChangeActivePage(Page: TMyTabSheet);
    procedure DeleteTab(Page: TMyTabSheet);
    function GetPage(Index: Integer): TMyTabSheet;
    function GetPageCount: Integer;
    procedure InsertPage(Page: TMyTabSheet);
    procedure InsertTab(Page: TMyTabSheet);
    procedure MoveTab(CurIndex, NewIndex: Integer);
    procedure RemovePage(Page: TMyTabSheet);
    procedure SetActivePage(Page: TMyTabSheet);
    procedure UpdateTab(Page: TMyTabSheet);
    procedure UpdateActivePage;
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure CMDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
  protected
    procedure Change; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    procedure ShowControl(AControl: TControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function FindNextPage(CurPage: TMyTabSheet;
      GoForward, CheckTabVisible: Boolean): TMyTabSheet;
    procedure SelectNextPage(GoForward: Boolean);
    property PageCount: Integer read GetPageCount;
    property Pages[Index: Integer]: TMyTabSheet read GetPage;
  published
    property ActivePage: TMyTabSheet read FActivePage write SetActivePage;
    property Align;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HotTrack;
    property MultiLine;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ScrollOpposite;
    property ShowHint;
    property TabHeight;
    property TabOrder;
    property TabPosition;
    property TabStop;
    property TabWidth;
    property Visible;
    property OnChange;
    property OnChanging;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;
var
  ControlForm: TControlForm;

implementation

{$R *.DFM}

uses Printers, Consts, ComStrs;

{ TTabSheet }

constructor TMyTabSheet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alClient;
  ControlStyle := ControlStyle + [csAcceptsControls, csNoDesignVisible];
  Visible := False;
  FTabVisible := True;
// Code added by BK to add labels and a stringgrid to the TabSheet.  15Dec97
  PnTop := TPanel.Create(Self);
  with PnTop do
   begin
     Parent := Self;
     Caption := '';
     Align := alTop
   end;
  LblProcessValue := TLabel.Create(Self);
  with LblProcessValue do
    begin
      Parent:=PnTop;
      Caption := '';
      Top := 14;
      Left := 182;
    end;
  LblProcessName := TLabel.Create(Self) ;
  with LblProcessName do
    begin
      Parent := PnTop;
      Caption := '';
      Top := 14;
      Left := 80;
    end;
  LblProcessUnits := TLabel.Create(Self) ;
  with LblProcessUnits do
    begin
      Parent := PnTop;
      Caption := '';
      Top := 14;
      Left := 310;
    end;
  SgParams := TStringGrid.Create(Self) ;
  with SgParams do
    begin
      Parent := Self;
      Align := AlClient;
      Height := 240;
      ColCount := 4;
      DefaultColWidth := 150;
      ColWidths[0] := 80;
      Cells[0,0] := 'Parameter';
      Cells[1,0] := 'Parameter Name';
      Cells[2,0] := 'Parameter Value';
      Cells[3,0] := 'Parameter Units';
    end;
// End of code added by BK.
end;

destructor TMyTabSheet.Destroy;
begin
  if FPageControl <> nil then FPageControl.RemovePage(Self);
  inherited Destroy;
end;

function TMyTabSheet.GetPageIndex: Integer;
begin
  if FPageControl <> nil then
    Result := FPageControl.FPages.IndexOf(Self) else
    Result := -1;
end;

function TMyTabSheet.GetTabIndex: Integer;
var
  I: Integer;
begin
  Result := 0;
  if not FTabShowing then Dec(Result) else
    for I := 0 to PageIndex - 1 do
      if TMyTabSheet(FPageControl.FPages[I]).FTabShowing then
        Inc(Result);
end;

procedure TMyTabSheet.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params.WindowClass do
    style := style and not (CS_HREDRAW or CS_VREDRAW);
end;

procedure TMyTabSheet.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TMyPageControl then
    PageControl := TMyPageControl(Reader.Parent);
end;

procedure TMyTabSheet.SetPageControl(APageControl: TMyPageControl);
begin
  if FPageControl <> APageControl then
  begin
    if FPageControl <> nil then FPageControl.RemovePage(Self);
    Parent := APageControl;
    if APageControl <> nil then APageControl.InsertPage(Self);
  end;
end;

procedure TMyTabSheet.SetPageIndex(Value: Integer);
var
  I, MaxPageIndex: Integer;
begin
  if FPageControl <> nil then
  begin
    MaxPageIndex := FPageControl.FPages.Count - 1;
    if Value > MaxPageIndex then
      raise EListError.CreateFmt(SPageIndexError, [Value, MaxPageIndex]);
    I := TabIndex;
    FPageControl.FPages.Move(PageIndex, Value);
    if I >= 0 then FPageControl.MoveTab(I, TabIndex);
  end;
end;

procedure TMyTabSheet.SetTabShowing(Value: Boolean);
begin
  if FTabShowing <> Value then
    if Value then
    begin
      FTabShowing := True;
      FPageControl.InsertTab(Self);
    end else
    begin
      FPageControl.DeleteTab(Self);
      FTabShowing := False;
    end;
end;

procedure TMyTabSheet.SetTabVisible(Value: Boolean);
begin
  if FTabVisible <> Value then
  begin
    FTabVisible := Value;
    UpdateTabShowing;
  end;
end;

procedure TMyTabSheet.UpdateTabShowing;
begin
  SetTabShowing((FPageControl <> nil) and FTabVisible);
end;

procedure TMyTabSheet.CMTextChanged(var Message: TMessage);
begin
  if FTabShowing then FPageControl.UpdateTab(Self);
end;

{ TPageControl }

constructor TMyPageControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csDoubleClicks, csOpaque];
  FPages := TList.Create;
end;

destructor TMyPageControl.Destroy;
var
  I: Integer;
begin
  for I := 0 to FPages.Count - 1 do TMyTabSheet(FPages[I]).FPageControl := nil;
  FPages.Free;
  inherited Destroy;
end;

procedure TMyPageControl.Change;
var
  Form: TCustomForm;
begin
  UpdateActivePage;
  if csDesigning in ComponentState then
  begin
    Form := GetParentForm(Self);
    if (Form <> nil) and (Form.Designer <> nil) then Form.Designer.Modified;
  end;
  inherited Change;
end;

procedure TMyPageControl.ChangeActivePage(Page: TMyTabSheet);
var
  ParentForm: TCustomForm;
begin
  if FActivePage <> Page then
  begin
    ParentForm := GetParentForm(Self);
    if (ParentForm <> nil) and (FActivePage <> nil) and
      FActivePage.ContainsControl(ParentForm.ActiveControl) then
      ParentForm.ActiveControl := FActivePage;
    if Page <> nil then
    begin
      Page.BringToFront;
      Page.Visible := True;
      if (ParentForm <> nil) and (FActivePage <> nil) and
        (ParentForm.ActiveControl = FActivePage) then
        if Page.CanFocus then
          ParentForm.ActiveControl := Page else
          ParentForm.ActiveControl := Self;
    end;
    if FActivePage <> nil then FActivePage.Visible := False;
    FActivePage := Page;
    if (ParentForm <> nil) and (FActivePage <> nil) and
      (ParentForm.ActiveControl = FActivePage) then
      FActivePage.SelectFirst;
  end;
end;

procedure TMyPageControl.DeleteTab(Page: TMyTabSheet);
begin
  Tabs.Delete(Page.TabIndex);
  UpdateActivePage;
end;

function TMyPageControl.FindNextPage(CurPage: TMyTabSheet;
  GoForward, CheckTabVisible: Boolean): TMyTabSheet;
var
  I, StartIndex: Integer;
begin
  if FPages.Count <> 0 then
  begin
    StartIndex := FPages.IndexOf(CurPage);
    if StartIndex = -1 then
      if GoForward then StartIndex := FPages.Count - 1 else StartIndex := 0;
    I := StartIndex;
    repeat
      if GoForward then
      begin
        Inc(I);
        if I = FPages.Count then I := 0;
      end else
      begin
        if I = 0 then I := FPages.Count;
        Dec(I);
      end;
      Result := FPages[I];
      if not CheckTabVisible or Result.TabVisible then Exit;
    until I = StartIndex;
  end;
  Result := nil;
end;

procedure TMyPageControl.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
begin
  for I := 0 to FPages.Count - 1 do Proc(TComponent(FPages[I]));
end;

function TMyPageControl.GetPage(Index: Integer): TMyTabSheet;
begin
  Result := FPages[Index];
end;

function TMyPageControl.GetPageCount: Integer;
begin
  Result := FPages.Count;
end;

procedure TMyPageControl.InsertPage(Page: TMyTabSheet);
begin
  FPages.Add(Page);
  Page.FPageControl := Self;
  Page.UpdateTabShowing;
end;

procedure TMyPageControl.InsertTab(Page: TMyTabSheet);
begin
  Tabs.InsertObject(Page.TabIndex, Page.Caption, Page);
  UpdateActivePage;
end;

procedure TMyPageControl.MoveTab(CurIndex, NewIndex: Integer);
begin
  Tabs.Move(CurIndex, NewIndex);
end;

procedure TMyPageControl.RemovePage(Page: TMyTabSheet);
begin
  if FActivePage = Page then SetActivePage(nil);
  Page.SetTabShowing(False);
  Page.FPageControl := nil;
  FPages.Remove(Page);
end;

procedure TMyPageControl.SelectNextPage(GoForward: Boolean);
var
  Page: TMyTabSheet;
begin
  Page := FindNextPage(ActivePage, GoForward, True);
  if (Page <> nil) and (Page <> ActivePage) and CanChange then
  begin
    TabIndex := Page.TabIndex;
    Change;
  end;
end;

procedure TMyPageControl.SetActivePage(Page: TMyTabSheet);
begin
  if (Page <> nil) and (Page.PageControl <> Self) then Exit;
  ChangeActivePage(Page);
  if Page <> nil then TabIndex := Page.TabIndex else TabIndex := -1;
end;

procedure TMyPageControl.SetChildOrder(Child: TComponent; Order: Integer);
begin
  TMyTabSheet(Child).PageIndex := Order;
end;

procedure TMyPageControl.ShowControl(AControl: TControl);
begin
  if (AControl is TMyTabSheet) and (TMyTabSheet(AControl).PageControl = Self) then
    SetActivePage(TMyTabSheet(AControl));
  inherited ShowControl(AControl);
end;

procedure TMyPageControl.UpdateTab(Page: TMyTabSheet);
begin
  Tabs[Page.TabIndex] := Page.Caption;
end;

procedure TMyPageControl.UpdateActivePage;
begin
  if TabIndex >= 0 then SetActivePage(TMyTabSheet(Tabs.Objects[TabIndex]));
end;

procedure TMyPageControl.CMDesignHitTest(var Message: TCMDesignHitTest);
var
  HitIndex: Integer;
  HitTestInfo: TTCHitTestInfo;
begin
  HitTestInfo.pt := SmallPointToPoint(Message.Pos);
  HitIndex := SendMessage(Handle, TCM_HITTEST, 0, Longint(@HitTestInfo));
  if (HitIndex >= 0) and (HitIndex <> TabIndex) then Message.Result := 1;
end;

procedure TMyPageControl.CMDialogKey(var Message: TCMDialogKey);
begin
  if (Message.CharCode = VK_TAB) and (GetKeyState(VK_CONTROL) < 0) then
  begin
    SelectNextPage(GetKeyState(VK_SHIFT) >= 0);
    Message.Result := 1;
  end else
    inherited;
end;
end.
 